@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import collections.abc
 import pathlib
 import re
 import sys
@@ -15,6 +16,7 @@ from typing import (
     Callable,
     Dict,
     Generic,
+    List,
     Mapping,
     Match,
     NewType,
@@ -55,6 +57,12 @@ Z = TypeVar("Z", bound="A")
 S = TypeVar("S", bound="miss")  # type: ignore # miss not defined on purpose # noqa: F821
 W = NewType("W", str)
 
+# Mypy does not support recursive type aliases, but
+# other type checkers do.
+RecList = Union[int, List["RecList"]]  # type: ignore
+MutualRecA = Union[bool, List["MutualRecB"]]  # type: ignore
+MutualRecB = Union[str, List["MutualRecA"]]  # type: ignore
+
 
 class A:
     def get_type(self) -> type:
@@ -146,6 +154,7 @@ def test_parse_annotation(annotation: Any, module: str, class_name: str, args: t
         (int, ":py:class:`int`"),
         (type(None), ":py:obj:`None`"),
         (type, ":py:class:`type`"),
+        (collections.abc.Callable, ":py:class:`~collections.abc.Callable`"),
         (Type, ":py:class:`~typing.Type`"),
         (Type[A], ":py:class:`~typing.Type`\\[:py:class:`~%s.A`]" % __name__),
         (Any, ":py:data:`~typing.Any`"),
@@ -228,7 +237,7 @@ def test_parse_annotation(annotation: Any, module: str, class_name: str, args: t
         (V, ":py:class:`~typing.TypeVar`\\(``V``, contravariant=True)"),
         (X, ":py:class:`~typing.TypeVar`\\(``X``, :py:class:`str`, :py:class:`int`)"),
         (Y, ":py:class:`~typing.TypeVar`\\(``Y``, bound= :py:class:`str`)"),
-        (Z, ":py:class:`~typing.TypeVar`\\(``Z``, bound= :py:class:`~test_sphinx_autodoc_typehints.A`)"),
+        (Z, ":py:class:`~typing.TypeVar`\\(``Z``, bound= A)"),
         (S, ":py:class:`~typing.TypeVar`\\(``S``, bound= miss)"),
         # ## These test for correct internal tuple rendering, even if not all are valid Tuple types
         # Zero-length tuple remains
@@ -279,6 +288,14 @@ def test_parse_annotation(annotation: Any, module: str, class_name: str, args: t
                 "...], :py:class:`~numpy.float64`]"
             ),
         ),
+        (
+            RecList,
+            (":py:data:`~typing.Union`\\[:py:class:`int`, :py:class:`~typing.List`\\[RecList]]"),
+        ),
+        (
+            MutualRecA,
+            (":py:data:`~typing.Union`\\[:py:class:`bool`, :py:class:`~typing.List`\\[MutualRecB]]"),
+        ),
     ],
 )
 def test_format_annotation(inv: Inventory, annotation: Any, expected_result: str) -> None:
